@@ -1,4 +1,4 @@
-/* $Id: SoftSession.h 4671 2011-03-31 08:59:37Z rb $ */
+/* $Id: SoftSession.h 6202 2012-03-07 10:43:44Z jakob $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -37,7 +37,7 @@
 #define SOFTHSM_SOFTSESSION_H 1
 
 #include "config.h"
-#include "pkcs11.h"
+#include "cryptoki.h"
 #include "SoftFind.h"
 #include "SoftDatabase.h"
 #include "SoftKeyStore.h"
@@ -49,7 +49,6 @@
 #include <botan/pk_keys.h>
 #include <botan/auto_rng.h>
 #include <botan/secmem.h>
-using namespace Botan;
 
 class SoftFind;
 class SoftDatabase;
@@ -73,27 +72,41 @@ class SoftSession {
     bool findInitialized;
 
     // Digest
-    Pipe *digestPipe;
+    Botan::Pipe *digestPipe;
     CK_ULONG digestSize;
     bool digestInitialized;
 
+    // Encrypt
+    Botan::PK_Encryptor *pkEncryptor;
+    bool encryptSinglePart;
+    CK_ULONG encryptSize;
+    bool encryptInitialized;
+
+    // Decrypt
+    Botan::PK_Decryptor *pkDecryptor;
+    bool decryptSinglePart;
+    CK_ULONG decryptSize;
+    bool decryptInitialized;
+
     // Sign
-    PK_Signer *pkSigner;
+    Botan::PK_Signer *pkSigner;
     bool signSinglePart;
     CK_ULONG signSize;
     bool signInitialized;
+    CK_MECHANISM_TYPE signMech;
+    CK_OBJECT_HANDLE signKey;
 
     // Verify
-    PK_Verifier *pkVerifier;
+    Botan::PK_Verifier *pkVerifier;
     bool verifySinglePart;
     CK_ULONG verifySize;
     bool verifyInitialized;
 
     // Key store
-    Public_Key* getKey(CK_OBJECT_HANDLE hKey);
+    Botan::Public_Key* getKey(CK_OBJECT_HANDLE hKey);
     SoftKeyStore *keyStore;
 
-    AutoSeeded_RNG *rng;
+    Botan::AutoSeeded_RNG *rng;
 
     SoftDatabase *db;
 
