@@ -1,4 +1,4 @@
-/* $Id: file.cpp 3903 2010-09-08 15:35:02Z rb $ */
+/* $Id: file.cpp 6202 2012-03-07 10:43:44Z jakob $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -90,7 +90,7 @@ CK_RV readConfigFile() {
     }
 
     // Get the second part of the line
-    char *dbPath = strtok(NULL, ":");
+    char *dbPath = strtok(NULL, "\0");
     if(dbPath == NULL) {
       continue;
     }
