@@ -1,4 +1,4 @@
-/* $Id: mechanisms.h 3957 2010-09-20 14:49:11Z rb $ */
+/* $Id: mechanisms.h 6202 2012-03-07 10:43:44Z jakob $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -29,7 +29,7 @@
 #ifndef SOFTHSM_MECHANISMS_H
 #define SOFTHSM_MECHANISMS_H 1
 
-#include "pkcs11.h"
+#include "cryptoki.h"
 
 // The number of supported mechanisms
 #define NR_SUPPORTED_MECHANISMS 15
